/*
******************* library atsu ***************************
	Several functions used in high atmosphere models

*/
#include "atmodels.h"

double temlo (double altu, double c[])
/*


	Purpose:

		The function temlo  evaluates  the  temperature at the
		local region.

	Inputs:
		altu
			altitude of the point, in km (90 to 2000).
		c[7]
			array that contains the parameters used  in
			the evaluation of equations 3 and 4, as gi-
			ven by Jacchia.

	Outputs:
		temlo
			local temperature as  defined  in equations
			3 and 4, in kelvin.

	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Valdemir Carrara       - INPE - S.J.Campos - Brazil

	Date:
		Apr. 1987              v. 1.0
		July 2011				C version
*/
{
	static double auxi, higo, higx, teml;
	const double zx = 125.0, zo = 90.0, to = 188.0;
	
	higx   = altu - zx;
	higo   = altu - zo;
	teml   = to;

	if (higo == 0.0) return teml;

	if (higx > 0.0)
	{
		teml  = c[6] + c[3]*atan(c[4]*higx + c[5]*higx*higx*higx);
	}
	else
	{
		auxi  = higx/higo;
		teml  = c[6] + c[0]*atan(c[1]*higx + c[2]*higx*auxi*auxi);
	}
	
	return teml;
}

void sealat (double tyfr, double sudc, double rlat, double altu, double al[])
/*

	Purpose:

		The subroutine sealat  obtains  the variations  on the

		number density of the atmosphere, due to the seazonal-
		                                             ---
		latitudinal effect.
		---
	
	Inputs:
		tyfr
			fraction of the tropic year, in  the  range
			0. to 1., starting on jan. 1st.
		sudc
			sun declination in radians (-pi to pi).
		rlat
			declination (geocentric  latitude)  of  the
			point, in radians (-pi to pi).
		altu
			geocentric altitude  in km,  in  the  range
			90. to 2000..

	Outputs:
		al[6]
			array containing  the  seazonal-latitudinal
			variations for the he (helium), o2 (molecu-
			lar oxygen), n2  (molecular  nitrogen),  ar
			(argon), o (atomic oxygen)  and  h  (atomic
			hydrogen) number density, respectively.

	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Valdemir Carrara       - INPE - S.J.Campos - Brazil
		Bento Silva de Matos   - INPE - S.J.Campos - Brazil

	Date:
		Apr. 1987              v. 1.0
		July 2011				C version
*/
{
	static double delz, dslm, dslt, esse, pcap, sila;
	static int i;
	const double pitw = 6.28318530718;
	const double cr[6] = {-0.79, 0.0, 0.0, 0.0, -.160, 0.0};
	
	sila   = sin(rlat);
	dslt   = sudc*sila/0.409157536545;
	delz   = altu - 91.;
	esse   = 0.0140*delz*exp(-0.00130*delz*delz);
	pcap   = sin(pitw*tyfr + 1.720);
	dslm   = _copysign(sila*sila*esse*pcap, rlat);

	for (i = 0; i < 6; i++) al[i] = dslt*cr[i] + dslm;
	
	return;
}

void semian (double tyfr, double altu, double *alco)
/*

	Purpose:
		The subroutine semian gives the correction factor alco

		for   the  atmosphere   number  density,  due  to  the

		semiannual effect.
		------

	Inputs:
		tyfr
			fraction of the tropic year, in  the  range
			0. to 1., starting on jan. 1st.
		altu
			geocentric altitude  in km,  in  the  range
			90. to 2000..

	Outputs:
		alco
			the semiannual variation of the  atmosphere
			number density.
	References:
		(1) Jacchia, L. G. "Thermospheric Temperature, Density
		    and Compostion: New Models." Cambridge,  Ma,  SAO,
			1977. (SAO Special Report No 375).

	Authors:
		Valdemir Carrara       - INPE - S.J.Campos - Brazil
		Bento Silva de Matos   - INPE - S.J.Campos - Brazil

	Date:
		Apr. 1987              v. 1.0
		July 2011				C version
*/
{
	static double auxi, foft, goft, tauc;
	const double pitw = 6.28318530718;

	auxi   = 0.040*altu*altu/1.e+04 + 0.05;
	foft   = auxi*exp(-0.25e-02*altu);
	auxi   = pow(0.5 + 0.5*sin(pitw*tyfr + 6.04), 1.65);
	tauc   = 0.0954*(auxi - 0.5) + tyfr;
	auxi   = sin(2.*pitw*tauc + 4.26)*(1. + 0.467*sin(pitw*tauc + 4.14));
	goft   = auxi*0.382 + 0.0284;
	*alco   = foft*goft;
	
	return;
}


double atanh(double x)
/*

	Purpose:

		The datanh function calculates the hyperbolic  tangent
		arc of an argument x, in double precision.

	Inputs:
		x
			hyperbolic tangent value (-1<x<1).

	Outputs:
		atanh
			hyperbolic tangent arc.

	Author:
		Valdemir Carrara  apr/87    v 1.0 (FORTRAN)
		Valdemir Carrara  july/11	C version
*/
{
	if (x >= 1.0) return 100;
	if (x <= -1.0) return -100;

	return log((1.0 + x)/(1.0 - x))/2.0;
}
